@@ -1,18 +1,39 @@
-# -*-Python-*-
-# Created by sciortinof at 31 Jul 2020  09:11
-
-'''
+"""
 Methods for neutral beam analysis, particularly in relation to impurity transport studies.
 These script collects functions that should be device-agnostic.
-'''
+"""
+# MIT License
+#
+# Copyright (c) 2021 Francesco Sciortino
+#
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
+#
+# The above copyright notice and this permission notice shall be included in all
+# copies or substantial portions of the Software.
+#
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
+
 import numpy as np
 import matplotlib.pyplot as plt
+
 plt.ion()
 from scipy.interpolate import RectBivariateSpline, interp1d, interp2d
 import copy, itertools
 
 from .janev_smith_rates import js_sigma
 from .plot_tools import get_ls_cycle
+from . import atomic
 
 
 def get_neutrals_fsa(neutrals, geqdsk, debug_plots=True):
@@ -54,25 +75,25 @@ def get_neutrals_fsa(neutrals, geqdsk, debug_plots=True):
 
     """
 
-    beams = neutrals['beams']
-    names = neutrals['names']
-    zz = neutrals['Z'] / 1e2  # cm --> m
-    rr = neutrals['R'] / 1e2  # cm --> m
+    beams = neutrals["beams"]
+    names = neutrals["names"]
+    zz = neutrals["Z"] / 1e2  # cm --> m
+    rr = neutrals["R"] / 1e2  # cm --> m
 
     if debug_plots:
         # for debugging/plotting:
-        RBBBS = geqdsk['RBBBS']
-        ZBBBS = geqdsk['ZBBBS']
+        RBBBS = geqdsk["RBBBS"]
+        ZBBBS = geqdsk["ZBBBS"]
         fig, ax = plt.subplots()
-        ax.contourf(rr, zz, neutrals[beams[0]]['n=0'][0].T)
-        ax.plot(RBBBS, ZBBBS, 'w--', lw=5)
-        ax.set_xlabel('R [m]')
-        ax.set_ylabel('Z [m]')
+        ax.contourf(rr, zz, neutrals[beams[0]]["n=0"][0].T)
+        ax.plot(RBBBS, ZBBBS, "w--", lw=5)
+        ax.set_xlabel("R [m]")
+        ax.set_ylabel("Z [m]")
 
     # simple way to find rhop at any combination of R,Z coords:
-    RHOpRZ = geqdsk['AuxQuantities']['RHOpRZ']
-    Rgrid = geqdsk['AuxQuantities']['R']  # m
-    Zgrid = geqdsk['AuxQuantities']['Z']  # m
+    RHOpRZ = geqdsk["AuxQuantities"]["RHOpRZ"]
+    Rgrid = geqdsk["AuxQuantities"]["R"]  # m
+    Zgrid = geqdsk["AuxQuantities"]["Z"]  # m
 
     # extrapolate beam to cover the entire 2D poloidal cross section,
     # setting all neutral populations to 0 outside of the simulated region
@@ -80,92 +101,105 @@ def get_neutrals_fsa(neutrals, geqdsk, debug_plots=True):
     for beam in beams:
         neutrals_ext[beam] = {}
         for n_level in [0, 1, 2]:
-            neutrals_ext[beam][f'n={n_level}'] = {}
+            neutrals_ext[beam][f"n={n_level}"] = {}
             for ii, name in enumerate(names):
-                dens = neutrals[beam][f'n={n_level}'][ii]
+                dens = neutrals[beam][f"n={n_level}"][ii]
 
-                f = interp2d(rr, zz, dens.T, kind='linear', bounds_error=False, fill_value=0.0)
+                f = interp2d(
+                    rr, zz, dens.T, kind="linear", bounds_error=False, fill_value=0.0
+                )
                 tmp = f(Rgrid, Zgrid)
 
                 tmp[tmp < 0] = 0.0
-                neutrals_ext[beam][f'n={n_level}'][name] = tmp
+                neutrals_ext[beam][f"n={n_level}"][name] = tmp
 
     if debug_plots:
         fig, ax = plt.subplots()
-        CS = ax.contourf(Rgrid, Zgrid, neutrals_ext[beam]['n=0']['fdens'])
+        CS = ax.contourf(Rgrid, Zgrid, neutrals_ext[beam]["n=0"]["fdens"])
         cbar = fig.colorbar(CS)
-        cbar.ax.set_ylabel(r'f density [$cm^{-3}$]')
-        ax.plot(RBBBS, ZBBBS, c='w', lw=5)
-        ax.axis('equal')
-        CS = plt.contour(Rgrid, Zgrid, RHOpRZ, np.linspace(0.0, 1.2, 13), c='w')
+        cbar.ax.set_ylabel(r"f density [$cm^{-3}$]")
+        ax.plot(RBBBS, ZBBBS, c="w", lw=5)
+        ax.axis("equal")
+        CS = plt.contour(Rgrid, Zgrid, RHOpRZ, np.linspace(0.0, 1.2, 13), c="w")
         plt.clabel(CS, inline=1, fontsize=14)
-        ax.set_xlabel('R [m]')
-        ax.set_ylabel('Z [m]')
+        ax.set_xlabel("R [m]")
+        ax.set_ylabel("Z [m]")
 
     # Get flux surface average quantities:
     neut_fsa = {}
-    rhop = neut_fsa['rhop'] = geqdsk['AuxQuantities']['RHOp']
-    neut_fsa['names'] = names
+    rhop = neut_fsa["rhop"] = geqdsk["AuxQuantities"]["RHOp"]
+    neut_fsa["names"] = names
 
     for beam in beams:
         neut_fsa[beam] = {}
 
         for n_level in [0, 1, 2]:
-            neut_fsa[beam][f'n={n_level}'] = {}
+            neut_fsa[beam][f"n={n_level}"] = {}
 
             for ii, name in enumerate(names):
-                dens = neutrals_ext[beam][f'n={n_level}'][name]
+                dens = neutrals_ext[beam][f"n={n_level}"][name]
 
                 # flux surface average
                 def avg_function(r, z):
                     return RectBivariateSpline(Zgrid, Rgrid, dens, kx=1, ky=1).ev(z, r)
 
-                neut_fsa[beam][f'n={n_level}'][name] = geqdsk['fluxSurfaces'].surfAvg(function=avg_function)
+                neut_fsa[beam][f"n={n_level}"][name] = geqdsk["fluxSurfaces"].surfAvg(
+                    function=avg_function
+                )
 
     for beam in beams:
         # store beam component energies
-        neut_fsa[beam]['f_energy'] = neutrals[beam]['f_energy']  # keV
-        neut_fsa[beam]['h_energy'] = neutrals[beam]['h_energy']  # keV
-        neut_fsa[beam]['t_energy'] = neutrals[beam]['t_energy']  # keV
-        neut_fsa[beam]['m_beam'] = neutrals[beam]['m_beam']  # amu
-    neut_fsa['m_bckg'] = copy.deepcopy(neutrals['m_bckg'])  # amu
+        neut_fsa[beam]["f_energy"] = neutrals[beam]["f_energy"]  # keV
+        neut_fsa[beam]["h_energy"] = neutrals[beam]["h_energy"]  # keV
+        neut_fsa[beam]["t_energy"] = neutrals[beam]["t_energy"]  # keV
+        neut_fsa[beam]["m_beam"] = neutrals[beam]["m_beam"]  # amu
+    neut_fsa["m_bckg"] = copy.deepcopy(neutrals["m_bckg"])  # amu
 
     if debug_plots:
         ls_cycle = get_ls_cycle()
 
         # plot FSA neutral densities
         fig = plt.figure()
-        fig.set_size_inches(12, 9, forward=True)
+        fig.set_size_inches(9, 6, forward=True)
         a1 = plt.subplot2grid((4, 4), (0, 0), rowspan=4, colspan=3)
         a2 = plt.subplot2grid((4, 4), (0, 3), rowspan=4, colspan=1)
         for beam in beams:
             for n_level in [0, 1, 2]:
-                for name in ['fdens', 'hdens', 'tdens', 'dcx+halo']:
-                    if name == 'dcx+halo':
-                        name = 'halo'  # simple renaming
+                for name in ["fdens", "hdens", "tdens", "dcx+halo"]:
+                    if name == "dcx+halo":
+                        name = "halo"  # simple renaming
                     lss = next(ls_cycle)
-                    a1.plot(rhop, neut_fsa[beam][f'n={n_level}'][name], lss, lw=2.0)
-                    a2.plot([], [], lss, lw=2.0, label=f'{beam} {name}, n={n_level+1}')
+                    a1.plot(rhop, neut_fsa[beam][f"n={n_level}"][name], lss, lw=2.0)
+                    a2.plot([], [], lss, lw=2.0, label=f"{beam} {name}, n={n_level+1}")
 
-        a1.set_xlabel(r'$\rho_{p}$')
-        a1.set_ylabel('FSA density')
-        a2.axis('off')
+        a1.set_xlabel(r"$\rho_{p}$")
+        a1.set_ylabel("FSA density")
+        a2.axis("off")
         a2.legend(fontsize=14)
         # fig.tight_layout()
 
     return neut_fsa
 
 
-
-def get_NBI_imp_cxr_q(neut_fsa, q, rhop_Ti, times_Ti, Ti_eV, include_fast=True, include_halo=True, debug_plots=False):
+def get_NBI_imp_cxr_q(
+    neut_fsa,
+    q,
+    rhop_kp,
+    times_kp,
+    Ti_eV,
+    ne_cm3,
+    include_fast=True,
+    include_halo=True,
+    debug_plots=False,
+):
     """Compute flux-surface-averaged (FSA) charge exchange recombination for a given impurity with
     neutral beam components, applying appropriate Maxwellian averaging of cross sections and
     obtaining rates in [:math:`s^-1`] units. This method expects all neutral components to be given in a
     dictionary with a structure that is independent of NBI model.
 
-    Note that while Ti may be time-dependent, with a time base given by times_Ti, the FSA
+    Note that while Ti and ne may be time-dependent, with a time base given by times_kp, the FSA
     neutrals are expected to be time-independent. Hence, the resulting CXR rates will only have
-    time dependence that reflects changes in Ti, but not the NBI.
+    time dependence that reflects changes in Ti and ne, but not the NBI.
 
     Parameters
     ----------
@@ -173,12 +207,14 @@ def get_NBI_imp_cxr_q(neut_fsa, q, rhop_Ti, times_Ti, Ti_eV, include_fast=True,
         Dictionary containing FSA neutral densities in the form that is output by :py:meth:`get_neutrals_fsa`.
     q : int or float
         Charge of impurity species
-    rhop_Ti : array-like
+    rhop_kp : array-like
         Sqrt of poloidal flux radial coordinate for Ti profiles.
-    times_Ti : array-like
+    times_kp : array-like
         Time base on which Ti_eV is given [s]. 
     Ti_eV : array-like
-        Ion temperature profile on the rhop_Ti, times_Ti bases.
+        Ion temperature profile on the rhop_kp, times_kp bases, in units of :math:`eV`.
+    ne_cm3 : array-like
+        Electron density profile on the rhop_kp, times_kp bases, in units of :math:`cm^{-3}`.
     include_fast : bool, optional
         If True, include CXR rates from fast NBI neutrals. Default is True. 
     include_halo : bool, optional
@@ -203,106 +239,131 @@ def get_NBI_imp_cxr_q(neut_fsa, q, rhop_Ti, times_Ti, Ti_eV, include_fast=True,
     For details on inputs and outputs, it is recommendeded to look at the internal plotting functions. 
 
     """
-    m_bckg = neut_fsa['m_bckg']  # amu
-    rhop = neut_fsa['rhop']
+    m_bckg = neut_fsa["m_bckg"]  # amu
+    rhop = neut_fsa["rhop"]
 
-    if len(times_Ti) == 1:
-        Ti = np.atleast_2d(interp1d(rhop_Ti, Ti_eV[:, 0] / 1e3, bounds_error=False, fill_value=3e-3)(rhop)).T  # keV
+    if len(times_kp) == 1:
+        Ti_keV_interp = np.atleast_2d(
+            interp1d(rhop_kp, Ti_eV[:, 0] / 1e3, bounds_error=False, fill_value=3e-3)(
+                rhop
+            )
+        ).T
+        ne_cm3_interp = np.atleast_2d(interp1d(rhop_kp, ne_cm3[:, 0])(rhop)).T
     else:
-        Ti = RectBivariateSpline(times_Ti, rhop_Ti, Ti_eV / 1e3)(times_Ti, rhop)  # keV
+        Ti_keV_interp = RectBivariateSpline(times_kp, rhop_kp, Ti_eV / 1e3)(
+            times_kp, rhop
+        )
+        ne_cm3_interp = RectBivariateSpline(times_kp, rhop_kp, ne_cm3)(times_kp, rhop)
 
     # collect rates for each energy component and excited state (ONLY fast neutrals here)
     rates = {}
-    rates['rhop'] = rhop
-    rates['times'] = times_Ti
-    rates['cxr_total'] = np.zeros((len(rhop), len(times_Ti)))
+    rates["rhop"] = rhop
+    rates["times"] = times_kp
+    rates["cxr_total"] = np.zeros((len(rhop), len(times_kp)))
 
     # setup dictionaries here in case include_fast=False
     for beam in beams:
         rates[beam] = {}
-        rates[beam]['cxr_total'] = np.zeros((len(rhop), len(times_Ti)))
+        rates[beam]["cxr_total"] = np.zeros((len(rhop), len(times_kp)))
         for n_level in [0, 1, 2]:
-            rates[beam][f'n={n_level}'] = {}
+            rates[beam][f"n={n_level}"] = {}
 
     if include_fast:
         # compute impurity recombination rate with fast neutral populations
         for beam in beams:
-            m_beam = neut_fsa[beam]['m_beam']  # amu
+            m_beam = neut_fsa[beam]["m_beam"]  # amu
             for n_level in [0, 1, 2]:
-                for comp in ['f', 'h', 't']:
-                    rates[beam][f'n={n_level}'][comp] = {}
+                for comp in ["f", "h", "t"]:
+                    rates[beam][f"n={n_level}"][comp] = {}
 
                     # energy of each beam component
-                    energy = neut_fsa[beam][f'{comp}_energy']  # keV
+                    energy = neut_fsa[beam][f"{comp}_energy"]  # keV
 
                     # create cross section function only as a function of energy/amu for Maxwellian average
-                    sigma_fun = lambda E_per_amu: js_sigma(E_per_amu, q, n1=n_level + 1, type='cx')  # cm^2
+                    sigma_fun = lambda E_per_amu: js_sigma(
+                        E_per_amu, q, n1=n_level + 1, type="cx"
+                    )  # cm^2
 
-                    rate = bt_rate_maxwell_average(sigma_fun, Ti, energy, m_bckg, m_beam, n_level + 1.0)  # .T
-                    rates[beam][f'n={n_level}'][comp] = rate * neut_fsa[beam][f'n={n_level}'][f'{comp}dens'][:, np.newaxis]
+                    rate = bt_rate_maxwell_average(
+                        sigma_fun, Ti_keV_interp, energy, m_bckg, m_beam, n_level + 1.0
+                    )  # .T
+                    rates[beam][f"n={n_level}"][comp] = (
+                        rate
+                        * neut_fsa[beam][f"n={n_level}"][f"{comp}dens"][:, np.newaxis]
+                    )
 
                     # we are eventually interested in the total:
-                    rates[beam]['cxr_total'] += rates[beam][f'n={n_level}'][comp]
+                    rates[beam]["cxr_total"] += rates[beam][f"n={n_level}"][comp]
 
             # sum over all beams:
-            rates['cxr_total'] += rates[beam]['cxr_total']
+            rates["cxr_total"] += rates[beam]["cxr_total"]
 
     if include_halo:
+        # fetch CCD file for W from ADAS and only pick appropriate charge (Rydberg assumption)
+        atom_data = atomic.get_atom_data("W", files=["ccd"])
+        alpha_cx_rates = atomic.interp_atom_prof(
+            atom_data["ccd"],
+            np.log10(ne_cm3_interp),
+            np.log10(Ti_keV_interp * 1e3),
+            x_multiply=False,
+        )  # cm^3/s
+        rate = alpha_cx_rates[:, q - 1, 0]  # no recombination of neutral stage
+
         # Now add recombination from halo neutrals (all thermal interactions) - n-unresolved for impurities
         for beam in beams:
-            m_beam = neut_fsa[beam]['m_beam']  # amu
+            m_beam = neut_fsa[beam]["m_beam"]  # amu
 
             for n_level in [0, 1, 2]:
-                # use Janev & Smith rates for halos too...
-                sigma_fun = lambda E_per_amu: js_sigma(E_per_amu, q, n1=n_level + 1.0, type='cx')  # cm^2
-
-                # Maxwell average using a "beam" at thermal energy (WRONG)
-                # rate1 = bt_rate_maxwell_average(sigma_fun, Ti, Ti, m_bckg, m_beam, n_level + 1.0)
-
-                # use proper thermal-thermal averaging
-                # the following function at the moment can only take time-independent Ti!
-                rate = tt_rate_maxwell_average(sigma_fun, Ti[:, 0], m_bckg, m_beam, n_level + 1.0)
-
-                # plt.figure()
-                # plt.plot(rhop, rate1, label='bt')
-                # plt.plot(rhop, rate, label='tt')
-                # plt.xlabel(r'$\rho_p$')
-                # plt.legend()
-
-                rates[beam][f'n={n_level}']['halo'] = rate[:, None] * neut_fsa[beam][f'n={n_level}']['dcx+halo'][:, None]
-                rates[beam]['cxr_total'] += rates[beam][f'n={n_level}']['halo']
-                rates['cxr_total'] += rates[beam][f'n={n_level}']['halo']
+                rates[beam][f"n={n_level}"]["halo"] = (
+                    rate[:, None] * neut_fsa[beam][f"n={n_level}"]["dcx+halo"][:, None]
+                )
+                rates[beam]["cxr_total"] += rates[beam][f"n={n_level}"]["halo"]
+                rates["cxr_total"] += rates[beam][f"n={n_level}"]["halo"]
 
     if debug_plots:
         ls_cycle = get_ls_cycle()
 
         fig = plt.figure()
-        fig.set_size_inches(12, 9, forward=True)
+        fig.set_size_inches(9, 6, forward=True)
         a1 = plt.subplot2grid((4, 4), (0, 0), rowspan=4, colspan=3)
         a2 = plt.subplot2grid((4, 4), (0, 3), rowspan=4, colspan=1)
 
         for beam in beams:
             for n_level in [0, 1, 2]:
                 if include_fast:
-                    for comp in ['f', 'h', 't']:
+                    for comp in ["f", "h", "t"]:
                         # time average to reduce number of lines
                         lss = next(ls_cycle)
-                        a1.plot(rhop, np.mean(rates[beam][f'n={n_level}'][comp], axis=-1), lss)
-                        a2.plot([], [], lss, lw=2.0, label=f'{beam} {comp}-energy, n={n_level+1}')
+                        a1.plot(
+                            rhop,
+                            np.mean(rates[beam][f"n={n_level}"][comp], axis=-1),
+                            lss,
+                        )
+                        a2.plot(
+                            [],
+                            [],
+                            lss,
+                            lw=2.0,
+                            label=f"{beam} {comp}-energy, n={n_level+1}",
+                        )
                 if include_halo:
                     lss = next(ls_cycle)
-                    a1.plot(rhop, np.mean(rates[beam][f'n={n_level}']['halo'], axis=-1), lss)
-                    a2.plot([], [], lss, lw=2.0, label=f'{beam} halo, n={n_level+1}')
+                    a1.plot(
+                        rhop, np.mean(rates[beam][f"n={n_level}"]["halo"], axis=-1), lss
+                    )
+                    a2.plot([], [], lss, lw=2.0, label=f"{beam} halo, n={n_level+1}")
 
-            a1.plot(rhop, np.mean(rates[beam]['cxr_total'], axis=-1), label=f'beam {beam} CXR total')
-            a2.plot([], [], label=f'beam {beam} CXR total')
+            lss = next(ls_cycle)
+            a1.plot(rhop, np.mean(rates[beam]["cxr_total"], axis=-1), lss)
+            a2.plot([], [], lss, label=f"beam {beam} CXR total")
 
-        a1.plot(rhop, rates['cxr_total'])
-        a2.plot([], [], label=f'CXR total')
-        a1.set_xlabel(r'$\rho_p$')
-        a1.set_ylabel(fr'CXR rate (q={q}) [$s^{{-1}}$]')
+        lss = next(ls_cycle)
+        a1.plot(rhop, rates["cxr_total"], lss)
+        a2.plot([], [], lss, label=f"CXR total")
+        a1.set_xlabel(r"$\rho_p$")
+        a1.set_ylabel(fr"CXR rate (q={q}) [$s^{{-1}}$]")
         a2.legend(fontsize=14)
-        a2.axis('off')
+        a2.axis("off")
         fig.tight_layout()
 
     return rates
@@ -320,7 +381,11 @@ def beam_grid(uvw_src, axis, max_radius=255.0):
     pos = uvw_src + 100 * axis
     rsrc = np.sqrt(uvw_src[0] ** 2 + uvw_src[1] ** 2)
     if rsrc < max_radius:
-        print("Source radius:{} cannot be less then max_radius:{}".format(rsrc, max_radius))
+        print(
+            "Source radius:{} cannot be less then max_radius:{}".format(
+                rsrc, max_radius
+            )
+        )
         raise ValueError()
     dis = np.sqrt(np.sum((uvw_src - pos) ** 2.0))
     beta = np.arcsin((uvw_src[2] - pos[2]) / dis)
@@ -361,7 +426,7 @@ def rotation_matrix(alpha, beta, gamma):
     R[2, 0] = -sb
     R[2, 1] = cb * sg
     R[2, 2] = cb * cg
-    
+
     return R
 
 
@@ -378,13 +443,21 @@ def uvw_xyz(u, v, w, origin, R):
     """
     u, v, w = np.atleast_1d(u), np.atleast_1d(v), np.atleast_1d(w)
     orig_shape = u.shape
-    order = 'C'
-    uvw = np.transpose(np.array([u.flatten(order=order), v.flatten(order=order), w.flatten(order=order)]))
+    order = "C"
+    uvw = np.transpose(
+        np.array(
+            [u.flatten(order=order), v.flatten(order=order), w.flatten(order=order)]
+        )
+    )
     uvw_shifted = np.transpose(uvw - origin[np.newaxis, :])
     basis = np.linalg.inv(R)
 
     xyz = np.dot(basis, uvw_shifted)
-    x, y, z = xyz[0, :].reshape(orig_shape, order='C'), xyz[1, :].reshape(orig_shape, order='C'), xyz[2, :].reshape(orig_shape, order='C')
+    x, y, z = (
+        xyz[0, :].reshape(orig_shape, order="C"),
+        xyz[1, :].reshape(orig_shape, order="C"),
+        xyz[2, :].reshape(orig_shape, order="C"),
+    )
     return x, y, z
 
 
@@ -401,22 +474,22 @@ def xyz_uvw(x, y, z, origin, R):
     """
     x, y, z = np.atleast_1d(x), np.atleast_1d(y), np.atleast_1d(z)
     orig_shape = x.shape
-    order = 'C'
-    xyz = np.array([x.flatten(order=order), y.flatten(order=order), z.flatten(order=order)])
+    order = "C"
+    xyz = np.array(
+        [x.flatten(order=order), y.flatten(order=order), z.flatten(order=order)]
+    )
 
     basis = R
 
     uvw = np.dot(basis, xyz) + origin[:, np.newaxis]
-    u, v, w = uvw[0, :].reshape(orig_shape, order='C'), uvw[1, :].reshape(orig_shape, order='C'), uvw[2, :].reshape(orig_shape, order='C')
+    u, v, w = (
+        uvw[0, :].reshape(orig_shape, order="C"),
+        uvw[1, :].reshape(orig_shape, order="C"),
+        uvw[2, :].reshape(orig_shape, order="C"),
+    )
     return u, v, w
 
 
-
-
-
-
-
-
 def bt_rate_maxwell_average(sigma_fun, Ti_keV, E_beam, m_bckg, m_beam, n_level):
     """Calculates Maxwellian reaction rate for a beam with atomic mass "m_beam", 
     energy "E_beam", firing into a target with atomic mass "m_bckg" and temperature "T".
@@ -481,10 +554,14 @@ def bt_rate_maxwell_average(sigma_fun, Ti_keV, E_beam, m_bckg, m_beam, n_level):
             Erel = u2_to_erel * u2
 
             sig = np.zeros_like(Erel)
-            mask = Erel >= dE  # no possible interaction below hydrogen ionization potential
+            mask = (
+                Erel >= dE
+            )  # no possible interaction below hydrogen ionization potential
             sig[mask] = sigma_fun(Erel[mask] / ared)
 
-            fr[:, :, j] = sig * np.sqrt(u2) * np.exp(-(vz[i] ** 2.0 + vr[j] ** 2.0)) * vr[j]
+            fr[:, :, j] = (
+                sig * np.sqrt(u2) * np.exp(-(vz[i] ** 2.0 + vr[j] ** 2.0)) * vr[j]
+            )
 
         fz[:, :, i] = scipy.integrate.simps(fr, vr, axis=-1)
 
@@ -495,8 +572,6 @@ def bt_rate_maxwell_average(sigma_fun, Ti_keV, E_beam, m_bckg, m_beam, n_level):
     return rate
 
 
-
-
 def tt_rate_maxwell_average(sigma_fun, Ti_keV, m_i, m_n, n_level):
     """Calculates Maxwellian reaction rate for an interaction between two thermal populations,
     assumed to be of neutrals (mass m_n) and background ions (mass m_i).
@@ -539,18 +614,23 @@ def tt_rate_maxwell_average(sigma_fun, Ti_keV, m_i, m_n, n_level):
     T_per_amu = np.maximum(Ti_keV, 1.0e-6) / m_n  # T_bar
 
     integrand = (
-        lambda erel: bt_rate_maxwell_average(sigma_fun, Ti_keV, erel, m_i, m_n, n_level) * (2.0 * m_n * erel) ** (-0.5) * np.exp(-erel / Ti_keV)
+        lambda erel: bt_rate_maxwell_average(sigma_fun, Ti_keV, erel, m_i, m_n, n_level)
+        * (2.0 * m_n * erel) ** (-0.5)
+        * np.exp(-erel / Ti_keV)
     )
 
     dE = (13.6e-3) / (n_level ** 2)  # hydrogen ionization potential
     sigma = np.zeros_like(Erel)
 
     for ie in np.arange(len(vz)):  # loop over vz
-        mask = Erel[:, ie] >= dE  # no possible interaction below hydrogen ionization potential
-        sigma[mask, ie] = bt_rate_maxwell_average(sigma_fun, Ti_keV[mask], Erel[mask, ie], m_i, m_n, n_level)
+        mask = (
+            Erel[:, ie] >= dE
+        )  # no possible interaction below hydrogen ionization potential
+        sigma[mask, ie] = bt_rate_maxwell_average(
+            sigma_fun, Ti_keV[mask], Erel[mask, ie], m_i, m_n, n_level
+        )
 
     prefactor = np.sqrt(2.0 / (np.pi * T_per_amu)) ** (-0.5)
     sigmav = prefactor * scipy.integrate.simps(sigma, Erel, axis=-1)
 
     return sigmav
-
